#include "getfem_mesh.h"
#include "getfem_mesh_fem.h"
#include "getfem_interpolation.h"
#include "getfem_assembling.h"
#include "getfem_interpolated_fem.h"

using namespace std;

int main () 
{
		//////////////////////////////////////////////////////////////////////////////////////////////
		//What's in here:
		//
		//This program will load 2 meshes from the disk. One is a coarse mesh, that we'll name 'mainmesh';
		//the other is a fine mesh, that we'll name 'atommesh'. The objective is to interpolate a 
		//vector of data that is defined on every node of the fine mesh into the nodes of the coarse mesh.
		//////////////////////////////////////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////////////////////////////
		//Opening the coarse mesh from the disk:
		std::string filename = "femmeshin.txt";
		getfem::mesh *mainmesh = new getfem::mesh();
		try
		{
			mainmesh->read_from_file(filename);
		}
		catch(exception e)
		{
			cout << "Error reading femmeshin.txt file. Program will now abort." << endl;
			terminate();
		};
		//Time to associate the shape functions to this mesh:
		getfem::mesh_fem mainmeshfem( *mainmesh );
		mainmeshfem.set_finite_element( getfem::fem_descriptor("FEM_PK(3,1)") );
		//And also the object that holds the FEM quadrature integration method:
		getfem::mesh_im integrationmethod( *mainmesh );
		integrationmethod.set_integration_method( getfem::int_method_descriptor("IM_TETRAHEDRON(2)") );
		//////////////////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////////////////////////////
		//Opening the fine mesh from the disk:
		getfem::mesh *atommesh = new getfem::mesh();
		filename = "./atommeshin.txt";
		try
		{
			atommesh->read_from_file(filename);
		}
		catch(exception e)
		{
			cout << "Error reading atommeshin.txt file. Program will now abort." << endl;
			terminate();
		};
		//Time to associate the shape functions to this mesh:
		getfem::mesh_fem atommeshfem( *atommesh );
		atommeshfem.set_finite_element( getfem::fem_descriptor("FEM_PK(3,1)") );
		//////////////////////////////////////////////////////////////////////////////////////////////
	
		//////////////////////////////////////////////////////////////////////////////////////////////
		//Now I'll create a vector that will represent the data on the nodes of the fine mesh:
		std::vector<double> data;	
		data.resize( atommeshfem.nb_dof() );
		unsigned int i = atommeshfem.nb_dof();
		do
		{
			i--;
			data[i] = 1.0; //Just some dummy value. You can use rand() here if you wish.
		}while(i);
		//////////////////////////////////////////////////////////////////////////////////////////////
	
		//////////////////////////////////////////////////////////////////////////////////////////////
		//So far, everything ok. Now we are going to try to interpolate the data from the fine to the
		//coarse mesh. Upon trying this block, the library will print the following message:
		//
		//'WARNING : in interpolation (different meshes),98 dof of target mesh_fem have been missed'
		//
		//Which is very very bad since our coarse mesh has only 124 points and we need all of them
		//interpolated:
		try
		{
		cout << "Trying the built in library function:\n";
		std::vector<double> interpolateddata;	
		interpolateddata.resize( mainmeshfem.nb_dof() );
		getfem::interpolation( atommeshfem, mainmeshfem, data , interpolateddata, true ); //the true flag triggers interpolation/extrapolation
		}
		DAL_STANDARD_CATCH_ERROR;
		//////////////////////////////////////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////////////////////////////
		//Ok, the initial idea was to use the previous block to solve the problem. Perhaps we could
		//try something a little different: instead of the builtin function, we could use the assembling
		//procedures to calculate the values we need. But since the getfem::mesh_fem objects are linked
		//to different meshes, first we need to build an interpolation. So when I was trying this out:
		cout << "Playing with the fem_interpolation:\n";
		try
		{
			getfem::mesh_fem atommeshfem_interpolator( atommeshfem.linked_mesh() );
			getfem::pfem interpolation = getfem::new_interpolated_fem( atommeshfem, integrationmethod);

			//The last line prints the following message:
			//
			//interpolated_fem::update from context : convex_index = [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383]'
			//
			//It's meaning is obscure to me! The fine mesh has more than 1000 dofs, so why do the first 384
			//dofs are printed out? Time to mail the experts... 
		}
		DAL_STANDARD_CATCH_ERROR;	
		//////////////////////////////////////////////////////////////////////////////////////////////
		
		return 0;
}


